
/******************************************************************************
* MODULE     : initialize.cc
* DESCRIPTION: Initialize the standard routines
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
#include <function.hh>
#include <single.hh>
#include <printer.hh>
#include <keyword.hh>
namespace mmx {

/******************************************************************************
* All keywords (multiple versions due to continuations)
* should be in same file as where the evaluator_rep constructor
******************************************************************************/

keyword KW_COMPOUND_1 ("compound!", &evaluator_rep::kw_compound_1);
keyword KW_COMPOUND_2 ("compound!2", &evaluator_rep::kw_compound_2);
keyword KW_COMPOUND_3 ("compound!3", &evaluator_rep::kw_compound_3);
keyword KW_EVAL_ARGS_1 ("eval_args!", &evaluator_rep::kw_eval_args_1);
keyword KW_LOAD_1 (MMX_LOAD, &evaluator_rep::kw_load_1);
keyword KW_LOAD_2 ("load!2", &evaluator_rep::kw_load_2);
keyword KW_USE_1 (MMX_USE, &evaluator_rep::kw_use_1);
keyword KW_USE_2 ("use!2", &evaluator_rep::kw_use_2);

keyword KW_CONV_AMBIGUOUS_1 (CONV_AMBIGUOUS,
			     &evaluator_rep::kw_conv_ambiguous_1);
keyword KW_CONV_APPLY_1 (CONV_APPLY, &evaluator_rep::kw_conv_apply_1);
keyword KW_CONV_APPLY_2 ("apply#2", &evaluator_rep::kw_conv_apply_2);
keyword KW_CONV_APPLY_3 ("apply#3", &evaluator_rep::kw_conv_apply_3);
keyword KW_CONV_CROSS_1 (CONV_CROSS, &evaluator_rep::kw_conv_cross_1);
keyword KW_CONV_CROSS_2 ("cross#2", &evaluator_rep::kw_conv_cross_2);
keyword KW_CONV_FORALL_1 (CONV_FORALL, &evaluator_rep::kw_conv_forall_1);
keyword KW_CONV_IGNORE_1 (CONV_IGNORE, &evaluator_rep::kw_conv_ignore_1);
keyword KW_CONV_LAMBDA_1 (CONV_LAMBDA, &evaluator_rep::kw_conv_lambda_1);
keyword KW_CONV_TEMPLATE_1 (CONV_TEMPLATE, &evaluator_rep::kw_conv_template_1);
keyword KW_CONV_TEMPLATE_2 ("template#2", &evaluator_rep::kw_conv_template_2);
keyword KW_CONV_TEMPLATE_3 ("template#3", &evaluator_rep::kw_conv_template_3);
keyword KW_CONV_UNALIAS_1 (CONV_UNALIAS, &evaluator_rep::kw_conv_unalias_1);
keyword KW_CONV_UNALIAS_2 ("unalias#2", &evaluator_rep::kw_conv_unalias_2);
keyword KW_CONV_UNAND_1 (CONV_UNAND, &evaluator_rep::kw_conv_unand_1);
keyword KW_CONV_UNAND_2 ("unand#2", &evaluator_rep::kw_conv_unand_2);
keyword KW_CONV_UNAND_3 ("unand#3", &evaluator_rep::kw_conv_unand_3);
keyword KW_CONV_UNCROSS_1 (CONV_UNCROSS, &evaluator_rep::kw_conv_uncross_1);
keyword KW_CONV_UNCROSS_2 ("uncross#2", &evaluator_rep::kw_conv_uncross_2);

keyword KW_ASSIGN_1 (MMX_ASSIGN, &evaluator_rep::kw_assign_1);
keyword KW_ASSIGN_2 (":=!2", &evaluator_rep::kw_assign_2);
keyword KW_ASSIGN_3 (":=!3", &evaluator_rep::kw_assign_3);
keyword KW_BEGIN_1 (MMX_BEGIN, &evaluator_rep::kw_begin_1);
keyword KW_BEGIN_2 ("begin!2", &evaluator_rep::kw_begin_2);
keyword KW_BREAK_1 (MMX_BREAK, &evaluator_rep::kw_break_1);
keyword KW_CONTINUE_1 (MMX_CONTINUE, &evaluator_rep::kw_continue_1);
keyword KW_CROSS_1 (MMX_CROSS, &evaluator_rep::kw_cross_1);
keyword KW_CROSS_2 ("Cross!2", &evaluator_rep::kw_cross_2);
keyword KW_CASE_1 (MMX_CASE, &evaluator_rep::kw_case_1);
keyword KW_CASE_2 ("case!2", &evaluator_rep::kw_case_2);
keyword KW_CASE_3 ("case!3", &evaluator_rep::kw_case_3);
keyword KW_CASE_4 ("case!4", &evaluator_rep::kw_case_4);
keyword KW_DEFINEALIAS_1 (MMX_DEFINEALIAS, &evaluator_rep::kw_define_1);
keyword KW_DEFINE_1 (MMX_DEFINE, &evaluator_rep::kw_define_1);
keyword KW_DEFINE_2 ("==!2", &evaluator_rep::kw_define_2);
keyword KW_DEFINE_3 ("==!3", &evaluator_rep::kw_define_3);
keyword KW_EXISTS_1 (MMX_EXISTS, &evaluator_rep::kw_quantor_1);
keyword KW_FOR_1 (MMX_FOR, &evaluator_rep::kw_for_1);
keyword KW_FORALL_1 (MMX_FORALL, &evaluator_rep::kw_quantor_1);
keyword KW_FORALLVAL_1 (MMX_FORALLVAL, &evaluator_rep::kw_forallval_1);
keyword KW_FORALLVAL_2 ("forallval!2", &evaluator_rep::kw_forallval_2);
keyword KW_FORIN_1 (MMX_FORIN, &evaluator_rep::kw_forin_1);
keyword KW_FORIN_2 ("forin!2", &evaluator_rep::kw_forin_2);
keyword KW_FORIN_3 ("forin!3", &evaluator_rep::kw_forin_3);
keyword KW_FORIN_4 ("forin!4", &evaluator_rep::kw_forin_4);
keyword KW_FORIN_5 ("forin!5", &evaluator_rep::kw_forin_5);
keyword KW_FORIN_6 ("forin!6", &evaluator_rep::kw_forin_6);
keyword KW_FUNCTION_1 (MMX_FUNCTION, &evaluator_rep::kw_function_1);
keyword KW_FUNCTION_2 ("->!2", &evaluator_rep::kw_function_2);
keyword KW_GENERATE_1 (MMX_GENERATE, &evaluator_rep::kw_generate_1);
keyword KW_GENERATE_2 ("generate!2", &evaluator_rep::kw_generate_2);
keyword KW_GENERATE_3 ("generate!3", &evaluator_rep::kw_generate_3);
keyword KW_IF_1 (MMX_IF, &evaluator_rep::kw_if_1);
keyword KW_IF_2 ("if!2", &evaluator_rep::kw_if_2);
keyword KW_JOIN_1 (MMX_JOIN, &evaluator_rep::kw_join_1);
keyword KW_JOIN_2 ("join!2", &evaluator_rep::kw_join_2);
keyword KW_JOIN_3 ("join!3", &evaluator_rep::kw_join_3);
keyword KW_LAMBDA_1 (MMX_LAMBDA, &evaluator_rep::kw_lambda_1);
keyword KW_LAMBDA_2 ("lambda!2", &evaluator_rep::kw_lambda_2);
keyword KW_LOCAL_1 (MMX_LOCAL, &evaluator_rep::kw_local_1);
keyword KW_LOCAL_2 ("local!2", &evaluator_rep::kw_local_2);
keyword KW_LOOP_1 (MMX_LOOP, &evaluator_rep::kw_loop_1);
keyword KW_LOOP_2 ("loop!2", &evaluator_rep::kw_loop_2);
keyword KW_LOOP_3 ("loop!3", &evaluator_rep::kw_loop_3);
keyword KW_LOOP_4 ("loop!4", &evaluator_rep::kw_loop_4);
keyword KW_QUANTOR_2 ("quantor!2", &evaluator_rep::kw_quantor_2);
keyword KW_RETRIEVE_1 (MMX_RETRIEVE, &evaluator_rep::kw_retrieve_1);
keyword KW_RETRIEVE_2 ("retrieve!2", &evaluator_rep::kw_retrieve_2);
keyword KW_RETRIEVE_3 ("retrieve!3", &evaluator_rep::kw_retrieve_3);
keyword KW_RETURN_1 (MMX_RETURN, &evaluator_rep::kw_return_1);
keyword KW_RETURN_2 ("return!2", &evaluator_rep::kw_return_2);
keyword KW_SPLIT_1 (MMX_SPLIT, &evaluator_rep::kw_split_1);
keyword KW_SPLIT_2 ("split!2", &evaluator_rep::kw_split_2);
keyword KW_STEP_1 (MMX_STEP, &evaluator_rep::kw_step_1);
keyword KW_TRANSTYPE_1 (MMX_TRANSTYPE, &evaluator_rep::kw_transtype_1);
keyword KW_TRANSTYPE_2 (":>!2", &evaluator_rep::kw_transtype_2);
keyword KW_UNTIL_1 (MMX_UNTIL, &evaluator_rep::kw_until_1);
keyword KW_WHILE_1 (MMX_WHILE, &evaluator_rep::kw_while_1);
keyword KW_YIELD_1 (MMX_YIELD, &evaluator_rep::kw_yield_1);
keyword KW_YIELD_2 ("yield!2", &evaluator_rep::kw_yield_2);

/******************************************************************************
* Some fancy test routines
******************************************************************************/

generic
test_print (generic g) {
  (void) g;
  cout << "Test\n";
  return generic ();
}

static int test_count= 10;

generic
test_bool (generic g) {
  (void) g;
  test_count--;
  return single (test_count >= 0);
}

/******************************************************************************
* Initialization
******************************************************************************/

void
evaluator_rep::declare (keyword kw) {
  env (kw->name)= kw;
}

evaluator_rep::evaluator_rep (): env (table ()) {
  initialize_master ();
  list types   = get_extern_types ();
  list bindings= get_extern_bindings ();
  (void) types;
  while (!nil (bindings)) {
    generic binding= car (bindings);
    declare (as_symbol (binding[0]), value (binding[1], binding[2]));
    bindings= cdr (bindings);
  }

  function test1 ("test_print", test_print);
  function test2 ("test_bool", test_bool);
  declare ("test_print", value (test1, function_type (ID_VOID)));
  declare ("test_bool", value (test2, function_type (ID_BOOL)));

  declare (KW_COMPOUND_1);
  declare (KW_COMPOUND_2);
  declare (KW_COMPOUND_3);
  declare (KW_EVAL_ARGS_1);
  declare (KW_LOAD_1);
  declare (KW_LOAD_2);
  declare (KW_USE_1);
  declare (KW_USE_2);

  declare (KW_CONV_AMBIGUOUS_1);
  declare (KW_CONV_APPLY_1);
  declare (KW_CONV_APPLY_2);
  declare (KW_CONV_APPLY_3);
  declare (KW_CONV_CROSS_1);
  declare (KW_CONV_CROSS_2);
  declare (KW_CONV_FORALL_1);
  declare (KW_CONV_IGNORE_1);
  declare (KW_CONV_LAMBDA_1);
  declare (KW_CONV_TEMPLATE_1);
  declare (KW_CONV_TEMPLATE_2);
  declare (KW_CONV_TEMPLATE_3);
  declare (KW_CONV_UNALIAS_1);
  declare (KW_CONV_UNALIAS_2);
  declare (KW_CONV_UNAND_1);
  declare (KW_CONV_UNAND_2);
  declare (KW_CONV_UNAND_3);
  declare (KW_CONV_UNCROSS_1);
  declare (KW_CONV_UNCROSS_2);

  //declare (KW_ASSIGN_1);
  //declare (KW_ASSIGN_2);
  //declare (KW_ASSIGN_3);
  declare (KW_BEGIN_1);
  declare (KW_BEGIN_2);
  declare (KW_BREAK_1);
  declare (KW_CONTINUE_1);
  declare (KW_CROSS_1);
  declare (KW_CROSS_2);
  declare (KW_CASE_1);
  declare (KW_CASE_2);
  declare (KW_CASE_3);
  declare (KW_CASE_4);
  declare (KW_DEFINEALIAS_1);
  declare (KW_DEFINE_1);
  declare (KW_DEFINE_2);
  declare (KW_DEFINE_3);
  declare (KW_EXISTS_1);
  declare (KW_FOR_1);
  declare (KW_FORALL_1);
  declare (KW_FORALLVAL_1);
  declare (KW_FORALLVAL_2);
  declare (KW_FORIN_1);
  declare (KW_FORIN_2);
  declare (KW_FORIN_3);
  declare (KW_FORIN_4);
  declare (KW_FORIN_5);
  declare (KW_FORIN_6);
  declare (KW_FUNCTION_1);
  declare (KW_FUNCTION_2);
  declare (KW_GENERATE_1);
  declare (KW_GENERATE_2);
  declare (KW_GENERATE_3);
  declare (KW_IF_1);
  declare (KW_IF_2);
  declare (KW_JOIN_1);
  declare (KW_JOIN_2);
  declare (KW_JOIN_3);
  declare (KW_LAMBDA_1);
  declare (KW_LAMBDA_2);
  declare (KW_LOCAL_1);
  declare (KW_LOCAL_2);
  declare (KW_LOOP_1);
  declare (KW_LOOP_2);
  declare (KW_LOOP_3);
  declare (KW_LOOP_4);
  declare (KW_QUANTOR_2);
  declare (KW_RETRIEVE_1);
  declare (KW_RETRIEVE_2);
  declare (KW_RETRIEVE_3);
  declare (KW_RETURN_1);
  declare (KW_RETURN_2);
  declare (KW_SPLIT_1);
  declare (KW_SPLIT_2);
  declare (KW_STEP_1);
  declare (KW_TRANSTYPE_1);
  declare (KW_TRANSTYPE_2);
  declare (KW_UNTIL_1);
  declare (KW_WHILE_1);
  declare (KW_YIELD_1);
  declare (KW_YIELD_2);
}

} // namespace mmx
