
/******************************************************************************
* MODULE     : evaluator.hh
* DESCRIPTION: the mathemagix evaluator
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __EVALUATOR_HH
#define __EVALUATOR_HH
#include <symbol.hh>
#include <table.hh>
#include <list.hh>
#include <value.hh>
#include <keyword.hh>
#include <extern.hh>
namespace mmx {

#ifdef DEBUG_ON
#define CHECK_SYNC(kw) \
  if (!is_func (car (trace), kw)) { \
    error (car (trace), "Lost synchronization"); \
    return; \
  }
#else
#define CHECK_SYNC(kw)
#endif

class value;
class evaluator_rep;
typedef void (evaluator_rep::*mmx_keyword) ();

class evaluator {
protected:
  evaluator_rep* rep;

public:
  inline evaluator ();
  inline evaluator (evaluator_rep* rep2): rep (rep2) {}
  inline friend evaluator as_evaluator (generic g) {
    if (nil (g)) return evaluator ();
    if (gentype (g) != EVALUATOR_TYPE) type_mismatch (g, "evaluator");
    return evaluator ((evaluator_rep*) g.rep); }
  inline operator generic ();
  inline evaluator_rep* operator -> () { return rep; }

  friend class evaluator_rep;
  friend ostream& operator << (ostream& out, evaluator con);
};

class evaluator_rep: generic_rep {
public:
  gentable env;   // the current environment
  generic  cur;   // next instruction to be interpreted
  list     rem;   // parameters for cur and remaining instructions
  list     trace; // original instructions being executed

public:
  evaluator_rep ();

  int  Type ();
  void Print (ostream& out);
  
  // declarations
  void declare (keyword kw);
  void declare (symbol var, value val);

  // evaluation
  generic loop ();
  generic evaluate (generic g);
  void    error (generic where, string message);

  void evaluate_arg (generic arg, keyword next);
  void evaluate_args (list args, keyword next);
  void evaluate_args (keyword next);

  // keywords
  void kw_compound_1 ();
  void kw_compound_2 ();
  void kw_compound_3 ();
  void kw_eval_args_1 ();
  void kw_load_1 ();
  void kw_load_2 ();
  void kw_use_1 ();
  void kw_use_2 ();

  void kw_conv_ambiguous_1 ();
  void kw_conv_apply_1 ();
  void kw_conv_apply_2 ();
  void kw_conv_apply_3 ();
  void kw_conv_cross_1 ();
  void kw_conv_cross_2 ();
  void kw_conv_forall_1 ();
  void kw_conv_ignore_1 ();
  void kw_conv_lambda_1 ();
  void kw_conv_template_1 ();
  void kw_conv_template_2 ();
  void kw_conv_template_3 ();
  void kw_conv_unalias_1 ();
  void kw_conv_unalias_2 ();
  void kw_conv_unand_1 ();
  void kw_conv_unand_2 ();
  void kw_conv_unand_3 ();
  void kw_conv_uncross_1 ();
  void kw_conv_uncross_2 ();

  void kw_assign_1 ();
  void kw_assign_2 ();
  void kw_assign_3 ();
  void kw_begin_1 ();
  void kw_begin_2 ();
  void kw_break_1 ();
  void kw_continue_1 ();
  void kw_cross_1 ();
  void kw_cross_2 ();
  void kw_case_1 ();
  void kw_case_2 ();
  void kw_case_3 ();
  void kw_case_4 ();
  void kw_define_1 ();
  void kw_define_2 ();
  void kw_define_3 ();
  void kw_for_1 ();
  void kw_forallval_1 ();
  void kw_forallval_2 ();
  void kw_forin_1 ();
  void kw_forin_2 ();
  void kw_forin_3 ();
  void kw_forin_4 ();
  void kw_forin_5 ();
  void kw_forin_6 ();
  void kw_function_1 ();
  void kw_function_2 ();
  void kw_generate_1 ();
  void kw_generate_2 ();
  void kw_generate_3 ();
  void kw_if_1 ();
  void kw_if_2 ();
  void kw_join_1 ();
  void kw_join_2 ();
  void kw_join_3 ();
  void kw_lambda_1 ();
  void kw_lambda_2 ();
  void kw_local_1 ();
  void kw_local_2 ();
  void kw_loop_1 ();
  void kw_loop_2 ();
  void kw_loop_3 ();
  void kw_loop_4 ();
  void kw_quantor_1 ();
  void kw_quantor_2 ();
  void kw_retrieve_1 ();
  void kw_retrieve_2 ();
  void kw_retrieve_3 ();
  void kw_return_1 ();
  void kw_return_2 ();
  void kw_split_1 ();
  void kw_split_2 ();
  void kw_step_1 ();
  void kw_transtype_1 ();
  void kw_transtype_2 ();
  void kw_until_1 ();
  void kw_while_1 ();
  void kw_yield_1 ();
  void kw_yield_2 ();

  friend class evaluator;
};

inline evaluator::evaluator (): rep (new evaluator_rep ()) {}
inline evaluator::operator generic () { return generic (rep); }

extern evaluator mmeval;
generic mmx_evaluate (generic g);

}
#endif // __EVALUATOR_HH
