
/******************************************************************************
* MODULE     : keyword.hh
* DESCRIPTION: mathemagix keywords
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __KEYWORD_HH
#define __KEYWORD_HH
#include <symbol.hh>
namespace mmx {

class evaluator_rep;
typedef void (evaluator_rep::*mmx_keyword) ();

/******************************************************************************
* The keyword class
******************************************************************************/

class keyword_rep;
class keyword {
protected:
  keyword_rep* rep;

public:
  inline keyword (symbol name, mmx_keyword fn);
  inline keyword (keyword_rep* rep2): rep (rep2) {}
  inline friend keyword as_keyword (generic g) {
    if (nil (g) || (gentype (g) != KEYWORD_TYPE)) type_mismatch (g, "keyword");
    return keyword ((keyword_rep*) g.rep); }
  inline operator generic ();
  inline keyword_rep* operator -> () { return rep; }

  friend class keyword_rep;
};

class keyword_rep: generic_rep {
public:
  symbol      name;
  mmx_keyword fn;

public:
  inline keyword_rep (symbol name2, mmx_keyword fn2):
    name (name2), fn (fn2) {}

  int  Type ();
  void Print (ostream& out);

  friend class keyword;
};

inline keyword::keyword (symbol name, mmx_keyword fn):
  rep (new keyword_rep (name, fn)) {}
inline keyword::operator generic () { return generic (rep); }

/******************************************************************************
* All keywords (multiple versions due to continuations)
******************************************************************************/

extern keyword KW_COMPOUND_1;
extern keyword KW_COMPOUND_2;
extern keyword KW_COMPOUND_3;
extern keyword KW_EVAL_ARGS_1;
extern keyword KW_LOAD_1;
extern keyword KW_LOAD_2;
extern keyword KW_USE_1;
extern keyword KW_USE_2;

extern keyword KW_CONV_AMBIGUOUS_1;
extern keyword KW_CONV_APPLY_1;
extern keyword KW_CONV_APPLY_2;
extern keyword KW_CONV_APPLY_3;
extern keyword KW_CONV_CROSS_1;
extern keyword KW_CONV_CROSS_2;
extern keyword KW_CONV_FORALL_1;
extern keyword KW_CONV_IGNORE_1;
extern keyword KW_CONV_LAMBDA_1;
extern keyword KW_CONV_TEMPLATE_1;
extern keyword KW_CONV_TEMPLATE_2;
extern keyword KW_CONV_TEMPLATE_3;
extern keyword KW_CONV_UNALIAS_1;
extern keyword KW_CONV_UNALIAS_2;
extern keyword KW_CONV_UNAND_1;
extern keyword KW_CONV_UNAND_2;
extern keyword KW_CONV_UNAND_3;
extern keyword KW_CONV_UNCROSS_1;
extern keyword KW_CONV_UNCROSS_2;

extern keyword KW_ASSIGN_1;
extern keyword KW_ASSIGN_2;
extern keyword KW_ASSIGN_3;
extern keyword KW_BEGIN_1;
extern keyword KW_BEGIN_2;
extern keyword KW_BREAK_1;
extern keyword KW_CONTINUE_1;
extern keyword KW_CROSS_1;
extern keyword KW_CROSS_2;
extern keyword KW_CASE_1;
extern keyword KW_CASE_2;
extern keyword KW_CASE_3;
extern keyword KW_CASE_4;
extern keyword KW_DEFINEALIAS_1;
extern keyword KW_DEFINE_1;
extern keyword KW_DEFINE_2;
extern keyword KW_DEFINE_3;
extern keyword KW_EXISTS_1;
extern keyword KW_FOR_1;
extern keyword KW_FORALL_1;
extern keyword KW_FORALLVAL_1;
extern keyword KW_FORALLVAL_2;
extern keyword KW_FORIN_1;
extern keyword KW_FORIN_2;
extern keyword KW_FORIN_3;
extern keyword KW_FORIN_4;
extern keyword KW_FORIN_5;
extern keyword KW_FORIN_6;
extern keyword KW_FUNCTION_1;
extern keyword KW_FUNCTION_2;
extern keyword KW_GENERATE_1;
extern keyword KW_GENERATE_2;
extern keyword KW_GENERATE_3;
extern keyword KW_IF_1;
extern keyword KW_IF_2;
extern keyword KW_JOIN_1;
extern keyword KW_JOIN_2;
extern keyword KW_JOIN_3;
extern keyword KW_LAMBDA_1;
extern keyword KW_LAMBDA_2;
extern keyword KW_LOCAL_1;
extern keyword KW_LOCAL_2;
extern keyword KW_LOOP_1;
extern keyword KW_LOOP_2;
extern keyword KW_LOOP_3;
extern keyword KW_LOOP_4;
extern keyword KW_QUANTOR_2;
extern keyword KW_RETRIEVE_1;
extern keyword KW_RETRIEVE_2;
extern keyword KW_RETRIEVE_3;
extern keyword KW_RETURN_1;
extern keyword KW_RETURN_2;
extern keyword KW_SPLIT_1;
extern keyword KW_SPLIT_2;
extern keyword KW_STEP_1;
extern keyword KW_TRANSTYPE_1;
extern keyword KW_TRANSTYPE_2;
extern keyword KW_UNTIL_1;
extern keyword KW_WHILE_1;
extern keyword KW_YIELD_1;
extern keyword KW_YIELD_2;

}
#endif // __KEYWORD_HH
