
/******************************************************************************
* MODULE     : glue.cc
* DESCRIPTION: glue for external libraries
* COPYRIGHT  : (C) 2003  Bernard Mourrain
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <glue.hh>

namespace mmx {

string
get_string (generic g) {
  //  cerr << ">>> " << g << "\n";
  return as_string (as_symbol (g));
}

string& operator<<(string & s, const symbol& u)
{
  string x= as_string(S(u));
  s<<x(1,N(s)-1);
  return s;
} 


table env;

//string&  operator<<(,symbol);
//static int  counter= 0;

bool
is_operator(generic fct)
{
  return(fct == MMX_PLUS || fct == MMX_TIMES || fct== MMX_MINUS 
	 || fct== MMX_OVER || fct == MMX_DOT);
}

bool
is_new_type(generic g)
{
  if(!is_list(g)) return false;
  generic t = car(cdr(g));
  if(nil(t)) return false;
  return(car(cdr(cdr(t))) == symbol("Type"));
}

bool
is_new_function(generic g)
{
  if(!is_list(g)) return false;
  generic t = car(cdr(g));
  if(nil(t)) return false;
  return(car(cdr(cdr(t))) != symbol("Type"));
}

bool
is_include(generic g)
{
  if(car(g) != MMX_DEFINE) return false;
  generic t = car(cdr(g));
  if(!is_list(t) || nil(t)) return false;
  return(car(cdr(t)) == symbol("Include"));
}

bool is_init(generic g)
{
  if(car(g) != MMX_DEFINE) return false;
  generic t = car(cdr(g));
  if(!is_list(t) || nil(t)) return false;
  return(car(cdr(t)) == symbol("Init"));
}

bool is_functional(generic g)
{
  //  cout<<">>> "<<g<<std::endl;
  if(!is_list(g)) return false;
  g =cdr(g);
  if(!is_list(g)) return false;  
  g =cdr(g);
  if(!is_list(g)) return false;
  g =car(g);
  if(!is_list(g)) return false;
  return (car(g) == MMX_FUNCTION);
}

generic define_new_type(generic g)
{
  //  std::cerr<<">>> New Type  "<<g<<std::endl;
  generic s = car(cdr(car(cdr(g))));
  env(s)   =  S(as_literal(car(cdr(cdr(g)))));
  //  std::cerr<<">>> New Type  "<<env(s)<<std::endl;
  return s;
}


void generate_new_type(std::fstream & fout, generic g)
{
  generic s=   define_new_type(g); 
  fout <<"mmx::symbol MMX_"<<s<<"(\""<<s<<"\"); "
       <<"mmx::mirror ID_"<<s<<"(MMX_"<<s<<");"<<std::endl;
}
void 
generate_include(std::fstream & fout, generic g)
{
  generic l = car(cdr(cdr(g)));
  l = cdr(l);
  while(!nil(l))
    { 
      string s= as_string(S(as_symbol(car(l))));
      s=s(1,N(s)-1);
      fout <<"#include <"<<as_charp(s)<<">"<<std::endl;;
      l =cdr(l);
    }
  fout <<std::endl;
}

void 
generate_init(std::fstream & fout, generic g)
{
  generic l = car(cdr(cdr(g)));
  l = cdr(l);
  while(!nil(l))
    { 
      fout <<as_charp(get_string(car(l)))<<";"<<std::endl;;
      l =cdr(l);
    }
  fout <<std::endl; 
}

int find(string s, string u)
{
  int i, j;
  for(i=0;i<N(s);i++)
    {
      for(j=0; i+j<N(s) && j<N(u) && s[i+j]==u[j] ;j++);
      if(j==N(u)) return i;
    }
  return -1;
}
void
generate_instruction (std::fstream & fout, generic v, generic out)
{
  //  cerr << ">>> Instruction " << v <<" "<< gentype (v) << lf;
  string s=  get_string(v);
  int i= find(s,"return ");
  if(i >= 0)
    {
      fout<<as_charp(s(0,i));
      s=s(i+7,N(s));
      int e=find(s,";");
      if(e>=0) s=s(0,e);
    }

  fout<<"return ";
  fout<<"mmx::mmx_obj< ";
  symbol y = as_symbol(env[out]); 
  fout << as_charp(as_string(y));
  fout <<" >(";
  fout<<as_charp(s);
  fout <<");"<<std::endl;

}
void 
generate_function_body(std::fstream & fout, generic w, generic out)
{
  symbol v= S (as_literal (w));
  //  cerr << ">>> Function " << v << " "<< gentype (v) << lf;
  if(!is_symbol(v)) {
       fout <<"Syntax error not a string!"<< v<<std::endl;
  } else 
  {
      fout <<"  ";
      if(out == symbol("Void")) 
	{
	  fout<<as_charp(get_string(v));
	  fout <<";\n  return _g;\n";
	}
      else
	generate_instruction(fout,v,out);
  }
}
void nameoffunction(std::fstream & fout, generic g)
{
  symbol s= as_symbol(g);
  if(s=="+")
    fout<<"add";
  else if(s=="-")
    fout<<"sub";
  else if(s=="*")
    fout<<"mul";
  else if(s=="/")
    fout<<"div";
  else if(s==".")
    fout<<"pnt";
  else
    fout<<s;
} 

void namefromvarsig(std::fstream & fout, generic g)
{
  generic l = cdr(car(cdr(g))); 
  fout << "mmx_fct_";
  nameoffunction(fout,car(car(cdr(g))));
  while(!nil(l))
    { 
      generic arg = cdr(car(l));
      generic typ = car(cdr(arg));
      fout<<"_"<< typ;
      l=cdr(l);
    }
  generic ot = car(cdr(cdr(g)));
  fout<<"__"<<ot;
}

void namefromfunctionalsig(std::fstream & fout, generic g)
{

  fout << "mmx_fct_";
  nameoffunction(fout,car(cdr(g)));
  generic l = car(cdr(cdr(g)));
  generic ot = car(cdr(cdr(l)));
  l = cdr(car(cdr(l)));
  while(!nil(l))
    { 
      generic typ = car(l);
      fout<<"_"<< typ;
      l=cdr(l);
    }
  fout<<"__"<<ot;
}

void namefromsig(std::fstream & fout, generic g)
{
  if(is_functional(g))
    namefromfunctionalsig(fout,g);
  else
    namefromvarsig(fout,g);
}


void generate_new_function(std::fstream & fout,generic g)
{
  //  cerr << ">>> New function " << g << " "<< gentype (g) << lf;
  //  std::cout<<">>> "<<g<<std::endl;
  generic sig = car(cdr(g));
  generic val = car(cdr(cdr(g)));
  //  std::cout<<"| "<<sig<<" "<<val<<std::endl;
  fout <<"\n//"<<sig
       << "\nmmx::generic ";
  namefromsig(fout,sig);
  fout<<"(mmx::generic _g)";
  int c;
  if(gentype (val)!=LIST_TYPE)
    {
      generic l = car(cdr(cdr(sig)));
      l = cdr(car(cdr(l))); c=1;
      fout <<"\n{"<<std::endl;
      //      cerr << ">>> Start body " << g << " " << gentype (g) << lf;
      while(!nil(l))
	{ 
	  generic typ = car(l);
	  symbol x= as_symbol(env[typ]);
	  fout<< "  "<<as_charp(as_string(x))<<" x"<<c
	      <<"(mmx::view_as< "<<as_charp(as_string(x))
	      <<" >(_g["<<c<<"]));"<<std::endl;
	  l=cdr(l); c++;
	}
      l = car(cdr(cdr(sig)));
      symbol fct=as_symbol(val);
      symbol typ=as_symbol(env[car(cdr(cdr(l)))]);
      fout<<"  return mmx_obj< "<<as_charp(as_string(typ)) <<" >("
	  <<as_charp(as_string(fct))<<"(";
      l = cdr(car(cdr(l))); 
      if(!nil(l)) fout<<"x1";
      l=cdr(l);     c=2;
      while(!nil(l))
	{
	  fout<<",x"<<c;
	  l=cdr(l);
	  c++;
	}
      fout<<"));"<<std::endl;
    }
  else 
    {
      generic l = car(cdr(val)); int c=1;
      fout <<"\n{"<<std::endl;
      while(!nil(l))
	{ 
	  generic arg = cdr(car(l));
	  generic typ = car(cdr(arg));
	  //	  std::cerr<<env[typ]<<std::endl;
	  symbol x= as_symbol(env[typ]);
	  fout<< "  "<<as_charp(as_string(x))<<" "<<car(arg)
	      <<"(mmx::view_as< "<<as_charp(as_string(x))
	      <<" >(_g["<<c<<"]));"<<std::endl;
	  l=cdr(l); c++;
	}
      sig = car(cdr(cdr(sig)));
      sig = car(cdr(cdr(sig)));
      val = car(cdr(cdr(cdr(val))));
      //      cout <<" -> "<<sig<<std::endl;
      generate_function_body(fout,val,sig);
    }
  fout <<"}"<<std::endl;
}


generic
generate (std::fstream & fout, generic g) {
  //  cout <<"generate "<<g<<std::endl;
  switch (gentype (g)) {
  case STRING_TYPE:
    {
      fout <<" -String- ";
      return value (g, MMX_STRING);
    } 
 case SYMBOL_TYPE:
    {
      symbol s= as_symbol (g);
      fout <<" -Symbol- "<<s;
    }
  case LIST_TYPE:
    {
      generic res= car(g);
      //   std::cout <<"LIST_TYPE "<<res<<" "<<gentype(res)<<std::endl;
      switch (gentype (res)) {
      case VALUE_TYPE:
	{
	  fout <<" VALUE ";
	  //	  value v= as_value (res);
	  generic args= generate (fout,cdr (as_list (g)));
	  fout <<args<<std::endl;
	  if (!is_list (args)) return args;
	  generate(fout,cdr (g));
	  break;
	} 
	//      case SYMBOL_TYPE:
	//	fout <<" Symbol "<<res<<std::endl;
      case SYMBOL_TYPE:
	{
	  symbol s= as_symbol(res);
	  if(s==MMX_LOCAL)
	    {
	      fout <<"\n//Start initialisation"<<std::endl;
	      g = cdr(g);
	      while(!nil(g))
		{
		  generate(fout,car (g));
		  g = cdr(g);
		}
	    }
	  else if(s==MMX_PREAMBLE)
	    {
	      fout <<car(cdr(g));
	    }
	  else if(s==MMX_DEFINE)
	    {
	      if(is_new_type(g))
		generate_new_type(fout,g);
	      else if(is_new_function(g))
		{
		  generate_new_function(fout,g);
		  //counter++;
		}
	    }
	  else if(s==MMX_TYPE)
	    {
	      fout <<"TYPE "<< g<<" ";
	    }
	  //	  else
	  //	    fout <<res;
	  break;
	}
      case KEYWORD_TYPE:
	cout <<"----> KEYWORD_TYPE"<<std::endl;
      default:
	{
	  generate(fout,res);
	  generate(fout,cdr (g));
	}
      }
    }
  }
  return g;
}
//----------------------------------------------------------------------
void 
link_new_type(std::fstream & fout,generic g)
{
  generic typ=car(cdr(car(cdr(g))));
  fout <<"  new_type(MMX_"<<typ<<", ID_"<<typ<<");\n";
}


void 
link_new_function(std::fstream & fout,generic g)
{
  fout <<"  new_function(\"";
  if(is_functional(g))
    fout<<car(cdr(g));
  else
    fout<<car(car(cdr(g)));
  fout<<"\",";
  namefromsig(fout,g); 
  fout<<",mmx::function_type(";
  if(is_functional(g))
    {
       generic l = car(cdr(cdr(g)));
       generic ot = car(cdr(cdr(l)));
       l = cdr(car(cdr(l)));
       while(!nil(l))
	 { fout<< "ID_"<<car(l)<<","; l=cdr(l);}
      fout << "ID_"<<ot<<"));"<<std::endl;
    }
  else {
    generic l = cdr(car(cdr(g))); 
    while(!nil(l))
      { fout<< "ID_"<<car(cdr(cdr(car(l))))<<","; l=cdr(l);}
    fout << "ID_"<<car(cdr(cdr(g)))<<"));"<<std::endl;
  }
}

generic
link (std::fstream & fout, generic g, string arg) {
  //  cout <<"link "<<g<<std::endl;
  switch (gentype (g)) {
  case STRING_TYPE:
    {
      fout <<" -String- ";
      return value (g, MMX_STRING);
    }
  case SYMBOL_TYPE:
   //  if (env->contains (g))
//       return env[g];
//     else 
    {
      symbol s= as_symbol (g);
      fout <<" -Symbol- "<<s;
      if (is_int (s))
	return value (single(as_int (s)), MMX_INT);
      else if (is_string (s))
	return value (as_string (s), MMX_STRING);
      //      else return exception (g, "Unknown identifier");
    }
  case SINGLE_TYPE:
    return value (g, MMX_INT);
  case LIST_TYPE:
    {
      generic res= (car (g));
      //      std::cout <<"LIST_TYPE "<<res<<" "<<gentype(res)<<std::endl;
      switch (gentype (res)) {
      case VALUE_TYPE:
	{
	  fout <<" VALUE ";
	  //	  value v= as_value (res);
	  generic args= link (fout,cdr (as_list (g)));
	  fout <<args<<std::endl;
	  if (!is_list (args)) return args;
	  link(fout,cdr (g));
	  break;
	} 
      case SYMBOL_TYPE:
	{
	  symbol s= as_symbol(res);
	  // cout <<g<<std::endl;
	  if(s==MMX_LOCAL)
	    {
	      fout <<"\nextern \"C\" void init_"<<as_charp(arg)<<"()\n{\n"
		   <<"  using namespace mmx;"<<std::endl;
	      link(fout,cdr (g),arg);
	      fout <<"};"<<std::endl;  
	    }
	  else if(s ==MMX_DEFINE)
	    {
	      if(is_include(g))
		{}
	      else if(is_init(g))
		{}
	      else if(is_new_type(g))
		link_new_type(fout,g);
	      else if(is_new_function(g))
		{
		  generic typ = car(cdr(g));
		  link_new_function(fout,typ);
		  //counter++;
		}
	    }
	  else if(s == MMX_TYPE)
	    {
	      fout <<"TYPE "<< g<<" ";
	    }
// 	  else
// 	    fout <<res;
	  break;
	}
      default:
	{
	  link(fout,res);
	  link(fout,cdr (g));
	}
      }
    }
  }
  return g;
}

//----------------------------------------------------------------------
string
input () {
  cout << "> ";
  fflush (stdout);
  char in[1000];
  cin.getline (in, 1000);
  return in;
}

void
parse_string (string s) {
  list errors;
  parse (s, errors);
  print_errors (errors);
  //  cout << output << "\n";
}
void init() 
{
  env(MMX_INT)=symbol(" int ");
  //  env(MMX_DOUBLE)=symbol(" double ");
  env(MMX_STRING)=symbol(" char* ");
  env(symbol("Void") )=symbol(" void ");
}

} // namespace mmx
