
/******************************************************************************
* MODULE     : xtable.cc
* DESCRIPTION: extended tables (for local environments)
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <xtable.hh>
namespace mmx {

/******************************************************************************
* Required functions for generic structures
******************************************************************************/

int
xtable_rep::Type () {
  return XTABLE_TYPE;
}

int
xtable_rep::Hash () {
  register int h= hash (base_t);
  h = (h<<11) + (h>>21);
  return h ^ hash (ext_t);
}

generic
xtable_rep::Copy () {
  return copy (xtable (this));
}

/******************************************************************************
* Routines for xtable representations
******************************************************************************/

generic
xtable_rep::get_binding (generic key) {
  generic binding= ext_t->get_binding (key);
  if (!nil (binding)) return binding;
  binding= base_t->get_binding (key);
  if (nil (binding)) return binding;
  ext_t->set_binding (binding);
  return binding;
}

void
xtable_rep::set_binding (generic binding) {
  ext_t->set_binding (binding);
}

void
xtable_rep::reset (generic key) {
  if (!ext_t->contains (key))
    fatal_error ("Can't reset key in base table",
		 "xtable_rep::reset", "xtable.cc");
  ext_t->reset (key);
}

list
xtable_rep::entries () {
  list l= ext_t->entries ();
  list b= base_t->entries ();
  while (!nil (b)) {
    if (!ext_t->contains (car (b) [0]))
      l= cons (car (b), l);
    b= cdr (b);
  }
  return reverse (l);
}

bool
xtable_rep::local_contains (generic key) {
  return ext_t->local_contains (key);
}

generic
xtable_rep::local_read (generic key) {
  return ext_t->local_read (key);
}

generic&
xtable_rep::local_write (generic key) {
  return ext_t->local_write (key);
}

int
xtable_rep::local_level () {
  return base_t->local_level () + 1;
}

/******************************************************************************
* Routines for xtables
******************************************************************************/

xtable
copy (xtable t) {
  return xtable (t->base_t, t->ext_t);
}

} // namespace mmx
